// GS1DataMatrixDLL.cpp : Definiert die exportierten Funktionen fr die DLL-Anwendung.
//
#include "stdafx.h"
//#include "GS1DataMatrix.h"
#include "pdfformfunc.h"
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
using namespace std;
#include <conio.h>
#include <ctype.h>

#define DATAMATRIXDLL_API

// F D F - Information
// spezial Feldertypen (cFDFspecialFields, Felder beginnen mit) : GS1DM_ und EAN128_ werden mit der Bayer-DLL auf Basis TEC-IT Software generiert ORG_EAN128_ ist der veryPDF Original EAN, der Fehler hat FNC1 nicht korrekt
const char* cFDFspecialFields[] = { "GS1DM_", "DM_", "BC128_", "EAN128_", "BC128RAW_", "BC39_", "EAN_", "Codabar_", "Inter25_", "PDF417_", "Postnet_", "EAN128VERYPDF_" };
// ... und ihre entsprechenden veryPDF-Prozedur-Aufrufe 
const char* cFDFspecialFieldsTypes[] = {"GS1DataMatrix", "Datamatrix", "Code128", "BayerEAN128", "Code128_RAW", "Code39", "EAN", "Codabar", "Inter25", "PDF417", "Postnet", "Code128_UCC"};

//  FDF-Feld Format
char* cFDFfieldNameStartsWith = "<</T(";
char* cFDFfieldValueStartsWith = "/V(";
char* cFDFfieldValueEndsWith = ")>>";


static string GetDefaultDirectory()
{
	char achDir[_MAX_PATH] = {0};
	GetCurrentDirectoryA(_MAX_PATH, achDir);

	return string(achDir);
}

static string CombinePath(const string& strDirectory, const string& strFileName)
{
	string strPath = strDirectory;
	if (!strPath.empty() && strPath[strPath.length() - 1] != '\\')
	{
		strPath += '\\';
	}
	strPath.append(strFileName);
	return strPath;
}

static string GetDirectory(string& path)
{
	char achDrive[_MAX_PATH] = {0};
	char achDir[_MAX_PATH] = {0};
	char fname[_MAX_PATH] = {0};
	char ext[_MAX_PATH] = {0};
	_splitpath(path.c_str(), achDrive, achDir, fname, ext );

	string strDir(achDrive);
	strDir += achDir;

	return strDir;
}

static string AppendFileName(string strFileName, const string& strSuffix)
{
	int extPos = strFileName.rfind(".");
	if (extPos >= 0)
	{
		strFileName.insert(extPos, strSuffix);
	}
	else
	{
		strFileName.append(strSuffix);
	}

	return strFileName;
}

static string ChangeFileExtension(string strFileName, const string& strExt)
{
	int extPos = strFileName.rfind(".");
	if (extPos >= 0)
	{
		strFileName.replace(extPos + 1, strFileName.length() - extPos - 1 , strExt);
	}
	else
	{
		strFileName += "." + strExt;
	}

	return strFileName;
}

static string GetTempFileName(string prefix)
{
	// Windows-Temp-Verzeichnis bestimmen
	char achPath[_MAX_PATH] = {0};
	::GetTempPathA(_MAX_PATH, achPath);
	string strPath = achPath;

	// temporren Dateinamen bestimmen
	::GetTempFileNameA(strPath.c_str(), prefix.c_str(), 0, achPath);
	strPath = CombinePath(strPath, string(achPath));

	return strPath;
} // GetTempFdfFileName(...)

static vector<string> GetAllPdfFields(string strPdfFile)
{
	vector<string> fields;

	char buffer[4096] = {0};
	int count = PDFForm_GetAllFields(strPdfFile.c_str(), buffer, sizeof(buffer) / sizeof(buffer[0]) - 1);
	string item(buffer);

	fields.push_back(item);
	return fields;
}

static string RemoveSpecialFieldsFromFdf(const string& strFdfContent, vector<string> logging)
{
	// Spezialfelder prfen und aus FDF-Inhalt entfernen
	string strFdfOut = strFdfContent;
	string feldInfos;
	for (int i = 0; i < sizeof(cFDFspecialFields) / sizeof(cFDFspecialFields[0]); i ++)
	{
		// das nchste Spezialfeld
		string strFdfField = cFDFspecialFields[i];
		string strFdfType = cFDFspecialFieldsTypes[i];

		// Feld in FDF-String suchen
		string strPattern(cFDFfieldNameStartsWith);
		strPattern += strFdfField;
		int fieldPos = strFdfOut.find(strPattern);
		while (fieldPos >= 0)
		{
			// vollstndigen Feldnamen bestimmen
			int namePos = fieldPos + ::strlen(cFDFfieldNameStartsWith);
			int nameEndPos = strFdfOut.find(")", namePos);
			string strFieldName = strFdfOut.substr(namePos, nameEndPos - namePos);

			// vollstndigen Wert bestimmen
			int valuePos = strFdfOut.find(cFDFfieldValueStartsWith, nameEndPos) + ::strlen(cFDFfieldValueStartsWith);
			int valueEndPos = strFdfOut.find(cFDFfieldValueEndsWith, valuePos);
			string strFieldValue = strFdfOut.substr(valuePos, valueEndPos - valuePos);

			// vollstndige Feldbeschreibung bestimmen
			int fieldEndPos = valueEndPos + ::strlen(cFDFfieldValueEndsWith);
			string strFieldDef = strFdfOut.substr(fieldPos, fieldEndPos - fieldPos);

			// Feld aus FDF-String entfernen
			strFdfOut.replace(fieldPos, fieldEndPos - fieldPos, "");
			logging.push_back(strFieldName + "=>" + strFdfType + "=>" + strFieldValue);

			// Weiteres Vorkommen dieses Spezialfeldes suchen
			fieldPos = strFdfOut.find(strPattern);
		}
	}

	return strFdfOut;
} // string RemoveSpecialFieldsFromFdf(...)

extern "C"
{


	/// <summary>
	/// Erzeugt eine Image-Datei, die die angegebenen Daten Barcode EAN128 darstellt.
	/// </summary>
	/// <param name="pszInput">FDF-Datei oder Dateiinhalt.
	/// </param>
	/// <param name="pszOutFile">Optional Dateiname der Ausgabedatei.</param>
	/// <returns>Error code:
	/// <para>0: OK</para>
	/// <para>13 (ERROR_INVALID_DATA): pszData contains invalid characters.</para>
    /// <para>24 (ERROR_BAD_LENGTH): pszData is too long.</para>
	/// <para>50 (ERROR_NOT_SUPPORTED): Image type is not supported.</para>
	/// <para>160 (ERROR_BAD_ARGUMENTS): Invalid argument</para>
	/// <para>1: Invalid argument</para>
	/// </returns>
	/// <remarks>
	/// </remarks>
	DATAMATRIXDLL_API int WINAPIV ExtendedFDF2PDF(LPCSTR pszInput, LPCSTR pszOutFile)
	{
		// Parameter prfen
		if (pszInput == NULL || ::strlen(pszInput) == 0)
		{
			// TODO: protokollieren
			return ERROR_BAD_ARGUMENTS;
		}

		// wenn pszInput ein Dateiname ist, FDF-Dateiinhalt einlesen
		string strWorkingDir = GetDefaultDirectory();
		string strFdfFileName;
		string strFdfContent;
		ifstream fdfStream;
		fdfStream.open(pszInput, ios_base::in);
		if (fdfStream.good())
		{
			// Dateinamen und Arbeitsverzeichnis abhngig von FDF-Datei festlegen
			strFdfFileName = pszInput;
			strWorkingDir = GetDirectory(strFdfFileName);

			// FDF-Inhalt aus Datei einlesen
			stringstream sstr;
			sstr << fdfStream.rdbuf();
			strFdfContent = sstr.str();
			fdfStream.close();
		}
		else
		{
			// direkte Eingabe der FDF-Datei
			strFdfFileName = "";
			strWorkingDir = GetDirectory(strFdfFileName);

			// FDF-Inhalt aus Parameter verwenden
			strFdfContent = pszInput;
		}

		// prfen, ob mindestens ein Dateiname fr Ein- oder Ausgabedatei vorhanden ist,
		// um Namen der Ausgabedatei bestimmen zu knnen
		string strPdfOutFile = pszOutFile;
		if (strFdfFileName.empty() && strPdfOutFile.empty())
		{
			// TODO: protokollieren
			return ERROR_BAD_ARGUMENTS;
		}

		// PDF-Bibliotheken initialisieren
		LPCSTR pszLicenseKey = "XXXXXXXXXXXXXXX";
		PDFForm_SetLicenseKey(pszLicenseKey);

		// wenn Dateiname der Zieldatei leer ist, aus Dateinamen der Eingabedatei bestimmen
		if (strPdfOutFile.empty())
		{
			strPdfOutFile = ChangeFileExtension(strFdfFileName, "pdf");
		}

        // prfen, ob PDF-Vorlage in FDF-Datei angegeben
        //   z. B. /F(80074913.pdf)/
		const char* pszPdfTemplatePattern = "<</F(";
		int nTemplatePos = strFdfContent.find(pszPdfTemplatePattern);
		if (nTemplatePos < 0)
		{
            // LoggingWindow(mFileFDF & " File-Beschreibung '<</F(' nicht gefunden - berprfen Sie die Syntax", activeInfo)
			return ERROR_INVALID_NAME;
		}

		// Dateinamen der PDF-Vorlage aus FDF-Inhalt bestimmen
		nTemplatePos += ::strlen(pszPdfTemplatePattern);
		int nTemplateEnd = strFdfContent.find(")", nTemplatePos);
		string strPdfTemplateFile = strFdfContent.substr(nTemplatePos, nTemplateEnd - nTemplatePos);

		// gegebenenfalls aktuellen Pfad ergnzen
		string strTemplateDir = GetDirectory(strPdfTemplateFile);
		if (strTemplateDir.empty())
		{
			strPdfTemplateFile = CombinePath(strWorkingDir, strPdfTemplateFile);
		}

		// Liste der Felder aus Vorlage auslesen
		vector<string> astrTemplateFields = GetAllPdfFields(strPdfTemplateFile);
		if (astrTemplateFields.empty())
		{
            // LoggingWindow(mFilePDFtemplate & " enthlt keine Felder", activeInfo)
			return ERROR_INVALID_DATA;
		}

		// Spezialfelder prfen und aus FDF-Inhalt entfernen
		vector<string> feldInfos;
		string strFdfOut = RemoveSpecialFieldsFromFdf(strFdfContent, feldInfos);

		// Info ber gelschte Felder ausgeben
		if (!feldInfos.empty())
		{
			for (vector<string>::iterator it = feldInfos.begin() ; it != feldInfos.end(); ++ it)
			{
				// TODO: Ausgabe it
			}
		}

		// Dateinamen der FDF-Datei ohne Spezialfelder bestimmen
		// bei Aufruf mit FDF-Dateiinhalt zuerst Namen einer temporren Datei festlegen
		string strFdfOutFile = strFdfFileName;
		if (strFdfOutFile.empty())
		{
			strFdfOutFile = GetTempFileName("Fdf");
			strFdfOutFile = ChangeFileExtension(strFdfOutFile, "fdf");
		}
		strFdfOutFile = AppendFileName(strFdfOutFile, "-OhneSpezialFelder");

		FILE* file = fopen(strFdfOutFile.c_str(), "wb");
		if(file)
		{
			fputs(strFdfOut.c_str(), file);
			fclose(file);
		}

		// Dateinamen fr temporre PDF-Ausgabedatei bestimmen
		// und reduzierte FDF-Datein in PDF-Template einfgen
		HRESULT result = E_FAIL;
		string strPdfTempFile = AppendFileName(strPdfOutFile, "_01");
		LPCSTR pszTemplate = strPdfTemplateFile.c_str();
		LPCSTR pszValues = strFdfOutFile.c_str();
		LPCSTR pszOut = strPdfTempFile.c_str();
		int retcode = PDFForm_MergeFDFIntoPDF(pszTemplate, pszValues, pszOut);
		if (0 != retcode)
		{
			result = NOERROR;
		}

		return result;
	} // int ExtendedFDF2PDF(...)
} // extern "C"

void GetModulePath(char *out_path,char *in_name)
{
	char *p;
	GetModuleFileName(NULL,out_path,256);
	p =strrchr(out_path,'\\');
	p[1]=0;
	strcat(out_path,in_name);
}

void main()
{
	char szFDFFile[MAX_PATH];
	GetModulePath(szFDFFile, "BXH0ZH0-001.fdf");
	char szPDFFile[MAX_PATH];
	GetModulePath(szPDFFile, "80074913.pdf");

	if(ExtendedFDF2PDF(szFDFFile, szPDFFile) == E_FAIL)
		printf("Failed to combine FDF into PDF file.\n");
	else
		printf("Combine FDF into PDF file successful.\n");
	_getch();
}